using at.jku.ssw.Analyse;

using System;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Collections;

namespace at.jku.ssw.Analyse.GUI
{
	/// <summary>
	/// Diese Klasse stellt allgemeine Funktionen fr alle ...InfoPanel-Klassen
	/// bereit.
	/// </summary>
	public class InfoPanel : UserControl
	{
		public virtual int getTabIndex() { return 0; }

		protected void FillVariableInfo(CodeInfo ci, ListView lv) {
			ListViewItem head = new ListViewItem(new String[] {"Variables:", ci.getCntVariables().ToString()});
			head.Font = new Font(head.Font, head.Font.Style | FontStyle.Bold);
			lv.Items.Add(head);
			foreach (VariableInfo vi in ci.declaredVariables)
				lv.Items.Add(new ListViewItem(new String[] {vi.sTyp, vi.sName}));
		}

		protected void FillLOCInfo(LinesOfCodeInfo loc, ListView lv) {
			ListViewItem head = new ListViewItem("Lines of Code Infos");
			head.Font = new Font(head.Font, head.Font.Style | FontStyle.Bold);
			lv.Items.AddRange(new ListViewItem[] {
													 head,
													 new ListViewItem(new String[] {"Lines of Code", loc.iLoCode.ToString()}),
													 new ListViewItem(new String[] {"Lines of Whitespace", loc.iLoWhitespace.ToString()}),
													 new ListViewItem(new String[] {"(Full) Lines of Comment", loc.getLoComment().ToString()}),
													 new ListViewItem(new String[] {"Total Lines", loc.iLines.ToString()}),
													 new ListViewItem(new String[] {"Lines of Code / Lines of Comment", loc.getLoCodeToLoComment().ToString()}),
													 new ListViewItem(new String[] {"Chars of Code", loc.iChCode.ToString()}),
													 new ListViewItem(new String[] {"Chars of Whitespace", loc.iChWhitespace.ToString()}),
													 new ListViewItem(new String[] {"Chars of Comment", loc.getChComment().ToString()}),
													 new ListViewItem(new String[] {"Total Chars", loc.iChars.ToString()}),
													 new ListViewItem(new String[] {"Chars of Code / Chars of Comment", loc.getChCodeToChComment().ToString()})
												 }
				);
		}

		protected void FillStatInfo(StatementCountInfo stat, ListView lv) {
			ListViewItem head = new ListViewItem("Statements Count Infos");
			head.Font = new Font(head.Font, head.Font.Style | FontStyle.Bold);
			lv.Items.AddRange(new ListViewItem[] {
													 head,
													 new ListViewItem(new String[] {"Statements", stat.iStatements.ToString()}),
													 new ListViewItem(new String[] {"Branches", stat.getCntBranches().ToString()}),
													 new ListViewItem(new String[] {"For-Loops", stat.getCntFor().ToString()}),
													 new ListViewItem(new String[] {"Foreach-Loops", stat.getCntForeach().ToString()}),
													 new ListViewItem(new String[] {"While-Loops", stat.getCntWhile().ToString()}),
													 new ListViewItem(new String[] {"Do-While-Loops", stat.getCntDo().ToString()})
												 }
				);
			head = new ListViewItem(new String[] {"Keywords:", stat.getCntKeywords().ToString()});
			head.Font = new Font(head.Font, head.Font.Style | FontStyle.Bold);
			lv.Items.Add(head);
			foreach (DictionaryEntry de in stat.htKeywords)
				lv.Items.Add(new ListViewItem(new String[] {((int)de.Value).ToString(), (String)de.Key}));
			head = new ListViewItem(new String[] {"Operators:", stat.getCntOperators().ToString()});
			head.Font = new Font(head.Font, head.Font.Style | FontStyle.Bold);
			lv.Items.Add(head);
			foreach (DictionaryEntry de in stat.htOperators)
				lv.Items.Add(new ListViewItem(new String[] {((int)de.Value).ToString(), (String)de.Key}));
		}

		protected void FillMethodAvgInfo(MethodAvgInfo mai, ListView lv) {
			lv.Items.AddRange(new ListViewItem[] {
					new ListViewItem(new String[] {"Method Count", mai.getCntMethods().ToString()}),
					new ListViewItem(new String[] {"Max. Parameters", mai.getMaxParameter().ToString()}),
					new ListViewItem(new String[] {"Avg. Parameters", mai.getAvgParameter().ToString()}),
					new ListViewItem(new String[] {"Max. Locals", mai.getMaxLocals().ToString()}),
					new ListViewItem(new String[] {"Avg. Locals", mai.getAvgLocals().ToString()}),
					new ListViewItem(new String[] {"Max. Depth", mai.getMaxDepth().ToString()}),
					new ListViewItem(new String[] {"Avg. Depth", mai.getAvgDepth().ToString()}),
					new ListViewItem(new String[] {"Max. McCabe", mai.getMaxMcCabe().ToString()}),
					new ListViewItem(new String[] {"Avg. McCabe", mai.getAvgMcCabe().ToString()}),
					new ListViewItem(new String[] {"Max. Henry&Kafura", mai.getMaxHenryKafura().ToString()}),
					new ListViewItem(new String[] {"Avg. Henry&Kafura", mai.getAvgHenryKafura().ToString()}),
					new ListViewItem(new String[] {"Max. Rechenberg", mai.getMaxRechenberg().ToString()}),
					new ListViewItem(new String[] {"Avg. Rechenberg", mai.getAvgRechenberg().ToString()})
				}
			);
		}

		protected void FillHalsteadInfos(HalsteadInfo hInfo, ListView lv) {
			ListViewItem head = new ListViewItem("Halstead's Infos");
			head.Font = new Font(head.Font, head.Font.Style | FontStyle.Bold);
			lv.Items.Add(head);
			lv.Items.AddRange(new ListViewItem[] {
					new ListViewItem(new String[] {"n1", hInfo.n1.ToString()}),
					new ListViewItem(new String[] {"n2", hInfo.n2.ToString()}),
					new ListViewItem(new String[] {"N1", hInfo.N1.ToString()}),
					new ListViewItem(new String[] {"N2", hInfo.N2.ToString()}),
					new ListViewItem(new String[] {"n (Vocabulary)", hInfo.n.ToString()}),
					new ListViewItem(new String[] {"N (Length)", hInfo.N.ToString()}),
					new ListViewItem(new String[] {"V (Volume)", hInfo.V.ToString()}),
					new ListViewItem(new String[] {"D (Difficulty)", hInfo.D.ToString()}),
					new ListViewItem(new String[] {"E (Effort)", hInfo.E.ToString()}),
					new ListViewItem(new String[] {"L (Level)", hInfo.L.ToString()}),
					new ListViewItem(new String[] {"T (Time)", hInfo.T.ToString()})
				}
			);

		}
	}
}
