using System;
using System.Text;

namespace at.jku.ssw.Analyse {
	/// <summary>
	/// Diese Klasse speichert alle Informationen, die mit der Anzahl der Zeilen und 
	/// Zeichen im Code zu tun haben.
	/// </summary>
	public class LinesOfCodeInfo {
		public int iLoCode, iLoWhitespace, iLines; // Anzahl der Leer-, Whitespace- und Gesamtzeilen
		public int iChCode, iChWhitespace, iChars; // Anzahl der Leer-, Whitespace- und Gesamtzeilen

		public LinesOfCodeInfo() {}

		/// <summary>
		/// Der + Operator ist notwendig, um LinesOfCode-Infos fr mehrere Bereiche
		/// zusammen addieren zu knnen (nach dem Ende eines Analyse-Durchlaufes).
		/// </summary>
		public static LinesOfCodeInfo operator+ (LinesOfCodeInfo loc1, LinesOfCodeInfo loc2) {
			LinesOfCodeInfo ret = new LinesOfCodeInfo();
			ret.iLoCode = loc1.iLoCode + loc2.iLoCode;
			ret.iLoWhitespace = loc1.iLoWhitespace + loc2.iLoWhitespace;
			ret.iLines = loc1.iLines + loc2.iLines;
			ret.iChCode = loc1.iChCode + loc2.iChCode;
			ret.iChWhitespace = loc1.iChWhitespace + loc2.iChWhitespace;
			ret.iChars = loc1.iChars + loc2.iChars;
			return ret;
		}

		/// <returns>Das Verhltnis LinesOfCode / LinesOfComment</returns>
		public float getLoCodeToLoComment() {
			if ((iLines - (iLoCode+iLoWhitespace)) == 0) return 0.0f;
			return (float)iLoCode / (float)(getLoComment());
		}

		/// <returns>Das Verhltnis CharsOfCode / CharsOfComment</returns>
		public float getChCodeToChComment() {
			if ((iChars - (iChCode+iChWhitespace)) == 0) return 0.0f;
			return (float)iChCode / (float)(getChComment());
		}

		/// <returns>Die Kommentarzeilen</returns>
		public int getLoComment() {
			return iLines - (iLoCode+iLoWhitespace);
		}

		/// <returns>Die Kommentarzeichen</returns>
		public int getChComment() {
			return iChars - (iChCode+iChWhitespace);
		}

		public String ToString(StringBuilder tab) {
			StringBuilder sb = new StringBuilder();
			sb.Append(tab).AppendFormat("Lines of Code:                    {0}\n", iLoCode);
			sb.Append(tab).AppendFormat("Lines of Whitespace:              {0}\n", iLoWhitespace);
			sb.Append(tab).AppendFormat("(Full) Lines of Comment:          {0}\n", getLoComment());
			sb.Append(tab).AppendFormat("Total Lines:                      {0}\n", iLines);
			sb.Append(tab).AppendFormat("Lines of Code / Lines of Comment: {0}\n", getLoCodeToLoComment());

			sb.Append(tab).AppendFormat("Chars of Code:                    {0}\n", iChCode);
			sb.Append(tab).AppendFormat("Chars of Whitespace:              {0}\n", iChWhitespace);
			sb.Append(tab).AppendFormat("Chars of Comment:                 {0}\n", getChComment());
			sb.Append(tab).AppendFormat("Total Chars:                      {0}\n", iChars);
			sb.Append(tab).AppendFormat("Chars of Code / Chars of Comment: {0}\n", getChCodeToChComment());

			return sb.ToString();
		}
	}
}
