using System;
using System.Text;

namespace at.jku.ssw.Analyse {
	/// <summary>
	/// Diese Klass stellt Methoden bereit, um den Durchschnitt und das Maximum
	/// einiger Kennzahlen berechnen knnen, die sich auf Methoden beziehen.
	/// </summary>
	public abstract class MethodAvgInfo : CodeInfo {
		public LinesOfCodeInfo locSumInfo; // aufsummierte LinesOfCodeInfo
		public StatementCountInfo statSumInfo; // aufsummierte StatementCountInfo

		public MethodAvgInfo(String sName, int iLine, CodeInfo parent) : base(sName, iLine, parent) {
			locSumInfo = new LinesOfCodeInfo();
			statSumInfo = new StatementCountInfo();
		}

		public int getCntMethods() {
			int ret = 0;
			foreach (CodeInfo ci in children) {
				if (typeof(MethodAvgInfo).IsInstanceOfType(ci)) ret += ((MethodAvgInfo) ci).getCntMethods();
				if (ci.GetType() == typeof(MethodInfo)) ret++;
			}
			return ret;
		}

		public int getMaxParameter() {
			int ret = 0;
			int tmp = 0;
			foreach (CodeInfo ci in children) {
				if (typeof(MethodAvgInfo).IsInstanceOfType(ci)) tmp = ((MethodAvgInfo) ci).getMaxParameter();
				if (ci.GetType() == typeof(MethodInfo)) tmp = ((MethodInfo) ci).getCntParameters();
				if (tmp > ret) ret = tmp;
			}
			return ret;
		}

		public float getAvgParameter() {
			float ret = 0.0f;
			int cnt = 0;
			foreach (CodeInfo ci in children) {
				if (typeof(MethodAvgInfo).IsInstanceOfType(ci)) {
					ret += ((MethodAvgInfo) ci).getCntMethods() * ((MethodAvgInfo) ci).getAvgParameter();
					cnt += ((MethodAvgInfo) ci).getCntMethods();
				}
				if (ci.GetType() == typeof(MethodInfo)) {
					ret += ((MethodInfo) ci).getCntParameters();
					cnt++;
				}
			}
			if (cnt != 0) return ret/(float)cnt;
			else return 0;
		}

		public int getMaxLocals() {
			int ret = 0;
			int tmp = 0;
			foreach (CodeInfo ci in children) {
				if (typeof(MethodAvgInfo).IsInstanceOfType(ci)) tmp = ((MethodAvgInfo) ci).getMaxLocals();
				if (ci.GetType() == typeof(MethodInfo)) tmp = ((MethodInfo) ci).getCntVariables();
				if (tmp > ret) ret = tmp;
			}
			return ret;
		}

		public float getAvgLocals() {
			float ret = 0.0f;
			int cnt = 0;
			foreach (CodeInfo ci in children) {
				if (typeof(MethodAvgInfo).IsInstanceOfType(ci)) {
					ret += ((MethodAvgInfo) ci).getCntMethods() * ((MethodAvgInfo) ci).getAvgLocals();
					cnt += ((MethodAvgInfo) ci).getCntMethods();
				}
				if (ci.GetType() == typeof(MethodInfo)) {
					ret += ((MethodInfo) ci).getCntVariables();
					cnt++;
				}
			}
			if (cnt != 0) return ret/(float)cnt;
			else return 0;
		}

		public int getMaxDepth() {
			int ret = 0;
			int tmp = 0;
			foreach (CodeInfo ci in children) {
				if (typeof(MethodAvgInfo).IsInstanceOfType(ci)) tmp = ((MethodAvgInfo) ci).getMaxDepth();
				if (ci.GetType() == typeof(MethodInfo)) tmp = ((MethodInfo) ci).iMaxDepth;
				if (tmp > ret) ret = tmp;
			}
			return ret;
		}

		public float getAvgDepth() {
			float ret = 0.0f;
			int cnt = 0;
			foreach (CodeInfo ci in children) {
				if (typeof(MethodAvgInfo).IsInstanceOfType(ci)) {
					ret += ((MethodAvgInfo) ci).getCntMethods() * ((MethodAvgInfo) ci).getAvgDepth();
					cnt += ((MethodAvgInfo) ci).getCntMethods();
				}
				if (ci.GetType() == typeof(MethodInfo)) {
					ret += ((MethodInfo) ci).iMaxDepth;
					cnt++;
				}
			}
			if (cnt != 0) return ret/(float)cnt;
			else return 0;
		}

		public int getMaxMcCabe() {
			int ret = 0;
			int tmp = 0;
			foreach (CodeInfo ci in children) {
				if (typeof(MethodAvgInfo).IsInstanceOfType(ci)) tmp = ((MethodAvgInfo) ci).getMaxMcCabe();
				if (ci.GetType() == typeof(MethodInfo)) tmp = ((MethodInfo) ci).iMcCabe;
				if (tmp > ret) ret = tmp;
			}
			return ret;
		}

		public float getAvgMcCabe() {
			float ret = 0.0f;
			int cnt = 0;
			foreach (CodeInfo ci in children) {
				if (typeof(MethodAvgInfo).IsInstanceOfType(ci)) {
					ret += ((MethodAvgInfo) ci).getCntMethods() * ((MethodAvgInfo) ci).getAvgMcCabe();
					cnt += ((MethodAvgInfo) ci).getCntMethods();
				}
				if (ci.GetType() == typeof(MethodInfo)) {
					ret += ((MethodInfo) ci).iMcCabe;
					cnt++;
				}
			}
			if (cnt != 0) return ret/(float)cnt;
			else return 0;
		}

		public float getMaxHenryKafura() {
			float ret = 0;
			float tmp = 0;
			foreach (CodeInfo ci in children) {
				if (typeof(MethodAvgInfo).IsInstanceOfType(ci)) tmp = ((MethodAvgInfo) ci).getMaxHenryKafura();
				if (ci.GetType() == typeof(MethodInfo)) tmp = ((MethodInfo) ci).fHenryKafura;
				if (tmp > ret) ret = tmp;
			}
			return ret;
		}

		public float getAvgHenryKafura() {
			float ret = 0.0f;
			int cnt = 0;
			foreach (CodeInfo ci in children) {
				if (typeof(MethodAvgInfo).IsInstanceOfType(ci)) {
					ret += ((MethodAvgInfo) ci).getCntMethods() * ((MethodAvgInfo) ci).getAvgHenryKafura();
					cnt += ((MethodAvgInfo) ci).getCntMethods();
				}
				if (ci.GetType() == typeof(MethodInfo)) {
					ret += ((MethodInfo) ci).fHenryKafura;
					cnt++;
				}
			}
			if (cnt != 0) return ret/(float)cnt;
			else return 0;
		}

		public float getMaxRechenberg() {
			float ret = 0;
			float tmp = 0;
			foreach (CodeInfo ci in children) {
				if (typeof(MethodAvgInfo).IsInstanceOfType(ci)) tmp = ((MethodAvgInfo) ci).getMaxRechenberg();
				if (ci.GetType() == typeof(MethodInfo)) tmp = ((MethodInfo) ci).fRechenbergCC;
				if (tmp > ret) ret = tmp;
			}
			return ret;
		}

		public float getAvgRechenberg() {
			float ret = 0.0f;
			int cnt = 0;
			foreach (CodeInfo ci in children) {
				if (typeof(MethodAvgInfo).IsInstanceOfType(ci)) {
					ret += ((MethodAvgInfo) ci).getCntMethods() * ((MethodAvgInfo) ci).getAvgRechenberg();
					cnt += ((MethodAvgInfo) ci).getCntMethods();
				}
				if (ci.GetType() == typeof(MethodInfo)) {
					ret += ((MethodInfo) ci).fRechenbergCC;
					cnt++;
				}
			}
			if (cnt != 0) return ret/(float)cnt;
			else return 0;
		}

		public override void compute() {
			if (!hasComputed) {
				base.compute();

				//die Summen der LinesOfCodeInfo und StatementInfo Informationen berechnen
				locSumInfo = locSumInfo + locInfo;
				statSumInfo = statSumInfo + statInfo;
				foreach(CodeInfo ci in children) {
					if (ci.GetType() == typeof(MethodInfo)) {
						locSumInfo = locSumInfo + ci.locInfo;
						statSumInfo = statSumInfo + ci.statInfo;
					}
					else {
						locSumInfo = locSumInfo + ((MethodAvgInfo)ci).locSumInfo;
						statSumInfo = statSumInfo + ((MethodAvgInfo)ci).statSumInfo;
					}
				}
			}
		}

		public override String ToString(String sTab, bool bToDescend) {
			StringBuilder sb = new StringBuilder();
			StringBuilder tab = new StringBuilder(sTab);
			sb.Append(base.ToString(sTab, bToDescend));
			sb.Append(tab).Append("Summed Lines of Codes - Informations:\n");
			sb.Append(locSumInfo.ToString(new StringBuilder(sTab+" ")));
			sb.Append(tab).Append("Summed Statement Count - Informations:\n");
			sb.Append(statSumInfo.ToString(new StringBuilder(sTab+" ")));
			sb.Append(tab).AppendFormat("Method Count:       {0}\n", getCntMethods());
			sb.Append(tab).AppendFormat("Max. Parameters:    {0}\n", getMaxParameter());
			sb.Append(tab).AppendFormat("Avg. Parameters:    {0}\n", getAvgParameter());
			sb.Append(tab).AppendFormat("Max. Locals:        {0}\n", getMaxLocals());
			sb.Append(tab).AppendFormat("Avg. Locals:        {0}\n", getAvgLocals());
			sb.Append(tab).AppendFormat("Max. Depth:         {0}\n", getMaxDepth());
			sb.Append(tab).AppendFormat("Avg. Depth:         {0}\n", getAvgDepth());
			sb.Append(tab).AppendFormat("Max. McCabe:        {0}\n", getMaxMcCabe());
			sb.Append(tab).AppendFormat("Avg. McCabe:        {0}\n", getAvgMcCabe());
			sb.Append(tab).AppendFormat("Max. Henry&Kafura:  {0}\n", getMaxHenryKafura());
			sb.Append(tab).AppendFormat("Avg. Henry&Kafura:  {0}\n", getAvgHenryKafura());
			sb.Append(tab).AppendFormat("Max. Rechenberg:    {0}\n", getMaxRechenberg());
			sb.Append(tab).AppendFormat("Avg. Rechenberg:    {0}\n", getAvgRechenberg());
			return sb.ToString();
		}

	}
}
