using System;
using System.Collections;
using System.Text;

namespace at.jku.ssw.Analyse
{
	/// <summary>
	/// Diese Klass ist eine Hilfsklasse zur Verwaltung von Namespaces und Aliases
	/// </summary>
	public class UsingInfo
	{
		public Hashtable htAliases; // Tabelle mit allen definierten Aliases
		public ArrayList alUsings; // Liste mit allen definierten Usings

		public UsingInfo()
		{
			htAliases = new Hashtable();
			alUsings = new ArrayList();
		}

		/// <summary>
		/// Wird vom Parser beim Auftreten eines using aufgerufen.
		/// </summary>
		public void addUsing(String name) {
			alUsings.Add(name);
		}

		/// <summary>
		/// Wird vom Parser beim Auftreten eines Aliases aufgerufen.
		/// </summary>
		/// <param name="name"></param>
		/// <param name="target"></param>
		public void addAlias(String name, String target) {
			htAliases[name] = target;
		}

		/// <summary>
		/// Diese Methode nimmt als Parameter name den Namen einer Klasse, Methode, etc.
		/// und versucht mit Hilfe der Informationen der Aliases und der Usings mgliche
		/// Namen zu erstellen, nach denen dann gesucht werden kann.
		/// </summary>
		/// <param name="name">Der Name der gesuchten Klasse, Methode, etc.</param>
		/// <param name="curNs">Der volle Name des aktuellen Objekts</param>
		/// <returns>Eine Liste mit allen Mglichkeiten</returns>
		public ArrayList getPossibilities(String name, String curNs) {
			ArrayList alPos = new ArrayList();
			// der Name allein ist immer eine Mglichkeit
			alPos.Add(name);
			// Vielleicht ist es sogar im gleichen Namespace?
			// Man ersetzt einfach den String hinter dem letzten Punkt
			int iTmp = curNs.LastIndexOf(".");
			if (iTmp != -1) alPos.Add(curNs.Substring(0, iTmp+1) + name);
			// Wenn ein Alias verwendet wurde, ersetzen wir ihn
			foreach (DictionaryEntry de in htAliases) {
				if (name.StartsWith((String)de.Key)) { //Alias gefunden
					name = de.Value + name.Substring(((String)de.Key).Length);
					alPos.Add(name);
					break;
				}
			}
			// Jetzt setzten wir noch alle mglichen Usings vor diesen String
			foreach (String s in alUsings) alPos.Add(s + "." + name);
			return alPos;
		}

		public String ToString(String sTab) {
			StringBuilder sb = new StringBuilder();
			sb.Append(sTab).AppendFormat("Usings:           {0}\n", alUsings.Count);
			foreach(String s in alUsings)
				sb.Append(sTab).Append(" ").AppendFormat("{0}\n", s);
			sb.Append(sTab).AppendFormat("Aliases:          {0}\n", htAliases.Count);
			foreach(DictionaryEntry de in htAliases)
				sb.Append(sTab).Append(" ").AppendFormat("{0} -> {1}\n", de.Key, de.Value);
			return sb.ToString();
		}
	}
}
